import { ContainerResolver } from '@adonisjs/fold';
import Macroable from '@poppinss/macroable';
import { Logger } from '@adonisjs/logger';
import { UrlWithStringQuery } from 'node:url';
import { Encryption } from '@adonisjs/encryption';
import { IncomingMessage, ServerResponse, IncomingHttpHeaders } from 'node:http';
import * as qs from 'qs';
import * as http from 'http';
import * as stream from 'stream';
import Middleware from '@poppinss/middleware';
import { NextFn, FinalHandler, ErrorHandler } from '@poppinss/middleware/types';
import { Readable } from 'node:stream';
import { Application } from '@adonisjs/application';

type QSParserConfig = {
    parse: {
        /**
         * Nesting depth till the parameters should be parsed.
         *
         * Defaults to 5
         */
        depth: number;
        /**
         * Number of parameters to parse.
         *
         * Defaults to 1000
         */
        parameterLimit: number;
        /**
         * Allow sparse elements in an array.
         *
         * Defaults to false
         */
        allowSparse: boolean;
        /**
         * The max limimit for the array indices. After the given limit
         * the array indices will be converted to an object, where the
         * index is the key.
         *
         * Defaults to 20
         */
        arrayLimit: number;
        /**
         * Join comma seperated query string values to an array
         *
         * Defaults to false
         */
        comma: boolean;
    };
    stringify: {
        /**
         * URI encode the stringified query string
         *
         * Defaults to true
         */
        encode: boolean;
        /**
         * URI encode but only the values and not the keys
         *
         * Defaults to false
         */
        encodeValuesOnly: boolean;
        /**
         * Define the format in which arrays should be serialized.
         *
         * - indices:   a[0]=b&a[1]=c
         * - brackets:  a[]=b&a[]=c
         * - repeat:    a=b&a=c
         * - comma:     a=b,c
         *
         * Defaults to "indices"
         */
        arrayFormat: 'indices' | 'brackets' | 'repeat' | 'comma';
        /**
         * Whether or not to skip null values when serializing. When set to
         * false, the null values will be treated as an empty string.
         *
         * Defaults to: false
         */
        skipNulls: boolean;
    };
};

/**
 * Query string parser used to parse and stringify query
 * strings.
 */
declare class Qs {
    #private;
    constructor(config: QSParserConfig);
    parse(value: string): qs.ParsedQs;
    stringify(value: any): string;
}

/**
 * Shape of the request config
 */
type RequestConfig = {
    /**
     * URL segments to ignore when extracting subdomains from a URL.
     * Defaults to 2
     */
    subdomainOffset: number;
    /**
     * Enabling the flag will generated a unique request id from every
     * HTTP request.
     *
     * The request id can be accessed using the "request.id()" method. Also,
     * the value of `x-request-id` header is used as the id (if it exists).
     *
     * Defaults to false
     */
    generateRequestId: boolean;
    /**
     * Method spoofing allows changing the request method using the query string.
     * For example: Making a POST request on URL /users/1?_method=PATCH will
     * be handled by the patch route.
     *
     * Defaults to false
     */
    allowMethodSpoofing: boolean;
    /**
     * A custom implementation to get the request ip address
     */
    getIp?: (request: any) => string;
    /**
     * A callback function to trust proxy ip addresses. You must use
     * the `proxy-addr` package to compute this value.
     *
     * Defaults to: "proxyAddr.compile('loopback')"
     */
    trustProxy: (address: string, distance: number) => boolean;
};

/**
 * Accept one or more of the mentioned type
 */
type OneOrMore<T> = T | T[];
/**
 * Class constructor type
 */
type Constructor<T> = new (...args: any[]) => T;
/**
 * A function that lazily imports a middleware
 */
type LazyImport<DefaultExport> = () => Promise<{
    default: DefaultExport;
}>;
/**
 * Unwraps default export from a lazy import function
 */
type UnWrapLazyImport<Fn extends LazyImport<any>> = Awaited<ReturnType<Fn>>['default'];

/**
 * Middleware represented as a class
 */
type MiddlewareAsClass = Constructor<{
    handle: (ctx: HttpContext, next: NextFn, args?: any) => any;
}>;
/**
 * Check if a union has undefined or null
 */
type HasUndefined<T> = T extends NonNullable<T> ? true : false;
/**
 * Returns the arguments accepted by the middleware's handle method
 */
type GetMiddlewareArgs<Middleware extends MiddlewareAsClass> = Parameters<InstanceType<Middleware>['handle']>[2] extends undefined ? [] : HasUndefined<Parameters<InstanceType<Middleware>['handle']>[2]> extends true ? [Parameters<InstanceType<Middleware>['handle']>[2]] : [Parameters<InstanceType<Middleware>['handle']>[2]?];
/**
 * The middleware defined as a function on the router or the server
 */
type MiddlewareFn = (ctx: HttpContext, next: NextFn) => any;
/**
 * Parsed global middleware
 */
type ParsedGlobalMiddleware = {
    handle: (resolver: ContainerResolver<any>, ...args: [ctx: HttpContext, next: NextFn, params?: any]) => any;
};
/**
 * Parsed named middleware
 */
type ParsedNamedMiddleware = ParsedGlobalMiddleware & {
    name: string;
    args: any;
};

/**
 * Cookie options can that can be set on the response
 */
type CookieOptions = {
    domain: string;
    expires: Date | (() => Date);
    httpOnly: boolean;
    maxAge: number | string;
    path: string;
    sameSite: boolean | 'lax' | 'none' | 'strict';
    secure: boolean;
};
/**
 * Types from which response header can be casted to a
 * string
 */
type CastableHeader = string | number | boolean | string[] | number[] | boolean[];
/**
 * Config accepted by response the class
 */
type ResponseConfig = {
    /**
     * Whether or not to generate etags for responses. Etags can be
     * enabled/disabled when sending response as well.
     *
     * Defaults to false
     */
    etag: boolean;
    /**
     * The callback name for the JSONP response.
     *
     * Defaults to 'callback'
     */
    jsonpCallbackName: string;
    /**
     * Options to set cookies
     */
    cookie: Partial<CookieOptions>;
};
/**
 * Stream that can be piped to the "response.stream" method
 */
type ResponseStream = Readable;

/**
 * Normalized HTTP error used by the exception
 * handler.
 */
type HttpError = {
    message: string;
    status: number;
    code?: string;
    stack?: string;
    cause?: any;
    messages?: any;
    errors?: any;
    handle?: (...args: any[]) => any;
    report?: (...args: any[]) => any;
};
/**
 * The pipeline for executing middleware during tests
 */
interface TestingMiddlewarePipeline {
    finalHandler(handler: FinalHandler): this;
    errorHandler(handler: ErrorHandler): this;
    run(ctx: HttpContext): Promise<any>;
}
/**
 * The expression to define a status page range
 */
type StatusPageRange = `${number}..${number}` | `${number}` | number;
/**
 * The callback function to render status page for a given
 * error.
 */
type StatusPageRenderer = (error: HttpError, ctx: HttpContext) => any | Promise<any>;
/**
 * Data type for the "http:request_finished" event
 */
type HttpRequestFinishedPayload = {
    ctx: HttpContext;
    duration: [number, number];
};
/**
 * Error handler to handle HTTP errors
 */
type ServerErrorHandler = {
    report: (error: any, ctx: HttpContext) => any;
    handle: (error: any, ctx: HttpContext) => any;
};
/**
 * Error handler represented as a class
 */
type ErrorHandlerAsAClass = Constructor<ServerErrorHandler>;
/**
 * Config accepted by the HTTP server
 */
type ServerConfig = RequestConfig & ResponseConfig & {
    /**
     * Whether or not to create an async local storage store for
     * the HTTP context.
     *
     * Defaults to false
     */
    useAsyncLocalStorage: boolean;
    /**
     * Config for query string parser
     */
    qs: QSParserConfig;
};

/**
 * Returns a union of methods from a controller that accepts
 * the context as the first argument.
 */
type GetControllerHandlers<Controller extends Constructor<any>> = {
    [K in keyof InstanceType<Controller>]: InstanceType<Controller>[K] extends (ctx: HttpContext, ...args: any[]) => any ? K : never;
}[keyof InstanceType<Controller>];
/**
 * Route token stored by matchit library
 */
type MatchItRouteToken = RouteMatcher & {
    old: string;
    type: 0 | 1 | 2 | 3;
    val: string;
    end: string;
};
/**
 * Route handler defined as a function
 */
type RouteFn = (ctx: HttpContext) => any;
/**
 * Route handler persisted with the route store
 */
type StoreRouteHandler = RouteFn | {
    reference: string | [LazyImport<Constructor<any>> | Constructor<any>, any?];
    handle: (resolver: ContainerResolver<any>, ...args: [ctx: HttpContext, ...injections: any[]]) => any;
};
/**
 * The middleware persisted with the route store
 */
type StoreRouteMiddleware = MiddlewareFn | ({
    name?: string;
    args?: any[];
} & ParsedGlobalMiddleware);
/**
 * Route node persisted within the routes store
 */
type StoreRouteNode = {
    /**
     * The execute function to execute the route middleware
     * and the handler
     */
    execute: (route: StoreRouteNode, resolver: ContainerResolver<any>, ctx: HttpContext, errorResponder: ServerErrorHandler['handle']) => any;
    /**
     * A unique name for the route
     */
    name?: string;
    /**
     * Route URI pattern
     */
    pattern: string;
    /**
     * Route handler
     */
    handler: StoreRouteHandler;
    /**
     * Route middleware
     */
    middleware: Middleware<StoreRouteMiddleware>;
    /**
     * Additional metadata associated with the route
     */
    meta: Record<string, any>;
};
/**
 * An object of routes for a given HTTP method
 */
type StoreMethodNode = {
    tokens: MatchItRouteToken[][];
    routeKeys: {
        [pattern: string]: string;
    };
    routes: {
        [pattern: string]: StoreRouteNode;
    };
};
/**
 * Each domain node container an object of methods. Each method
 * object has nested routes.
 */
type StoreDomainNode = {
    [method: string]: StoreMethodNode;
};
/**
 * Routes tree stored within the routes store
 */
type StoreRoutesTree = {
    tokens: MatchItRouteToken[][];
    domains: {
        [domain: string]: StoreDomainNode;
    };
};
/**
 * Shape of the matched route for a pattern, method and domain.
 */
type MatchedRoute = {
    route: StoreRouteNode;
    /**
     * A unique key for the looked up route
     */
    routeKey: string;
    /**
     * Route params
     */
    params: Record<string, any>;
    /**
     * Route subdomains (if part of a subdomain)
     */
    subdomains: Record<string, any>;
};
/**
 * Shape of a route param matcher
 */
type RouteMatcher = {
    match?: RegExp;
    cast?: (value: string) => any;
};
/**
 * A collection of route matchers
 */
type RouteMatchers$1 = {
    [param: string]: RouteMatcher;
};
/**
 * Representation of a route as JSON
 */
type RouteJSON = StoreRouteNode & {
    /**
     * HTTP methods, the route responds to.
     */
    methods: string[];
    /**
     * The domain for which the route is registered.
     */
    domain: string;
    /**
     * Matchers for route params.
     */
    matchers: RouteMatchers$1;
};
/**
 * Resource action names
 */
type ResourceActionNames = 'create' | 'index' | 'store' | 'show' | 'edit' | 'update' | 'destroy';
/**
 * Options accepted by makeUrl method
 */
type MakeUrlOptions = {
    qs?: Record<string, any>;
    domain?: string;
    prefixUrl?: string;
    disableRouteLookup?: boolean;
};
/**
 * Options accepted by makeSignedUrl method
 */
type MakeSignedUrlOptions = MakeUrlOptions & {
    expiresIn?: string | number;
    purpose?: string;
};

/**
 * The route class exposes the APIs for constructing a route using the
 * fluent API.
 */
declare class Route<Controller extends Constructor<any> = any> extends Macroable {
    #private;
    constructor(app: Application<any>, routerMiddleware: ParsedGlobalMiddleware[], options: {
        pattern: string;
        methods: string[];
        handler: RouteFn | string | [LazyImport<Controller> | Controller, GetControllerHandlers<Controller>?];
        globalMatchers: RouteMatchers$1;
    });
    /**
     * Define matcher for a given param. If a matcher exists, then we do not
     * override that, since the routes inside a group will set matchers
     * before the group, so they should have priority over the group
     * matchers.
     *
     * ```ts
     * Route.group(() => {
     *   Route.get('/:id', 'handler').where('id', /^[0-9]$/)
     * }).where('id', /[^a-z$]/)
     * ```
     *
     * The `/^[0-9]$/` will win over the matcher defined by the group
     */
    where(param: string, matcher: RouteMatcher | string | RegExp): this;
    /**
     * Define prefix for the route. Calling this method multiple times
     * applies multiple prefixes in the reverse order.
     */
    prefix(prefix: string): this;
    /**
     * Define a custom domain for the route. We do not overwrite the domain
     * unless `overwrite` flag is set to true.
     */
    domain(domain: string, overwrite?: boolean): this;
    /**
     * Define one or more middleware to be executed before the route
     * handler.
     *
     * Named middleware can be referenced using the name registered with
     * the router middleware store.
     */
    use(middleware: OneOrMore<MiddlewareFn | ParsedNamedMiddleware>): this;
    /**
     * @alias use
     */
    middleware(middleware: OneOrMore<MiddlewareFn | ParsedNamedMiddleware>): this;
    /**
     * Give a unique name to the route. Assinging a new unique removes the
     * existing name of the route.
     *
     * Setting prepends to true prefixes the name to the existing name.
     */
    as(name: string, prepend?: boolean): this;
    /**
     * Check if the route was marked to be deleted
     */
    isDeleted(): boolean;
    /**
     * Mark route as deleted. Deleted routes are not registered
     * with the route store
     */
    markAsDeleted(): void;
    /**
     * Get the route name
     */
    getName(): string | undefined;
    /**
     * Get the route pattern
     */
    getPattern(): string;
    /**
     * Set the route pattern
     */
    setPattern(pattern: string): this;
    /**
     * Returns the stack of middleware registered on the route.
     * The value is shared by reference.
     */
    getMiddleware(): StoreRouteMiddleware[][];
    /**
     * Returns JSON representation of the route
     */
    toJSON(): RouteJSON;
}

/**
 * Brisk routes exposes the API to configure the route handler by chaining
 * one of the pre-defined methods.
 *
 * For example: Instead of defining the redirect logic as a callback, one can
 * chain the `.redirect` method.
 *
 * Brisk routes are always registered under the `GET` HTTP method.
 */
declare class BriskRoute extends Macroable {
    #private;
    /**
     * Reference to route instance. Set after `setHandler` is called
     */
    route: null | Route;
    constructor(app: Application<any>, routerMiddleware: ParsedGlobalMiddleware[], options: {
        pattern: string;
        globalMatchers: RouteMatchers$1;
    });
    /**
     * Set handler for the brisk route
     */
    setHandler(handler: RouteFn): Route;
    /**
     * Redirects to a given route. Params from the original request will
     * be used when no custom params are defined.
     */
    redirect(identifier: string, params?: any[] | Record<string, any>, options?: MakeUrlOptions & {
        status: number;
    }): Route;
    /**
     * Redirect request to a fixed URL
     */
    redirectToPath(url: string, options?: {
        status: number;
    }): Route;
}

/**
 * Route resource exposes the API to register multiple routes for a resource.
 */
declare class RouteResource extends Macroable {
    #private;
    /**
     * A collection of routes instances that belongs to this resource
     */
    routes: Route[];
    constructor(app: Application<any>, routerMiddleware: ParsedGlobalMiddleware[], options: {
        resource: string;
        controller: string | LazyImport<Constructor<any>> | Constructor<any>;
        globalMatchers: RouteMatchers$1;
        shallow: boolean;
    });
    /**
     * Register only given routes and remove others
     */
    only(names: ResourceActionNames[]): this;
    /**
     * Register all routes, except the one's defined
     */
    except(names: ResourceActionNames[]): this;
    /**
     * Register api only routes. The `create` and `edit` routes, which
     * are meant to show forms will not be registered
     */
    apiOnly(): this;
    /**
     * Define matcher for params inside the resource
     */
    where(key: string, matcher: RouteMatcher | string | RegExp): this;
    /**
     * Tap into multiple routes to configure them by their name
     */
    tap(callback: (route: Route) => void): this;
    tap(actions: ResourceActionNames | ResourceActionNames[], callback: (route: Route) => void): this;
    /**
     * Set the param name for a given resource
     */
    params(resources: {
        [resource: string]: string;
    }): this;
    /**
     * Prepend name to all the routes
     */
    as(name: string, normalizeName?: boolean): this;
}

/**
 * Group class exposes the API to take action on a group of routes.
 * The group routes must be pre-defined using the constructor.
 */
declare class RouteGroup extends Macroable {
    #private;
    routes: (Route | RouteGroup | RouteResource | BriskRoute)[];
    constructor(routes: (Route | RouteGroup | RouteResource | BriskRoute)[]);
    /**
     * Define route param matcher
     *
     * ```ts
     * Route.group(() => {
     * }).where('id', /^[0-9]+/)
     * ```
     */
    where(param: string, matcher: RouteMatcher | string | RegExp): this;
    /**
     * Define prefix all the routes in the group.
     *
     * ```ts
     * Route.group(() => {
     * }).prefix('v1')
     * ```
     */
    prefix(prefix: string): this;
    /**
     * Define domain for all the routes.
     *
     * ```ts
     * Route.group(() => {
     * }).domain(':name.adonisjs.com')
     * ```
     */
    domain(domain: string): this;
    /**
     * Prepend name to the routes name.
     *
     * ```ts
     * Route.group(() => {
     * }).as('version1')
     * ```
     */
    as(name: string): this;
    /**
     * Prepend an array of middleware to all routes middleware.
     *
     * ```ts
     * Route.group(() => {
     * }).use(middleware.auth())
     * ```
     */
    use(middleware: OneOrMore<MiddlewareFn | ParsedNamedMiddleware>): this;
    /**
     * @alias use
     */
    middleware(middleware: OneOrMore<MiddlewareFn | ParsedNamedMiddleware>): this;
}

/**
 * Route finder is used to find a route by its name, route pattern
 * or the controller.method name.
 */
declare class RouteFinder {
    #private;
    constructor(routes: RouteJSON[]);
    /**
     * Find a route by indentifier
     */
    find(routeIdentifier: string): RouteJSON | null;
    /**
     * Find a route by indentifier or fail
     */
    findOrFail(routeIdentifier: string): RouteJSON;
    /**
     * Find if a route exists
     */
    has(routeIdentifier: string): boolean;
}

/**
 * URL builder class is used to create URIs for pre-registered
 * routes.
 *
 * ```ts
 * const builder = new UrlBuilder(encryption, routeFinder)
 *
 * builder
 *  .qs({ sort: 'id' })
 *  .params([category.id])
 *  .make('categories.posts.index')
 * ```
 */
declare class UrlBuilder {
    #private;
    constructor(encryption: Encryption, routeFinder: RouteFinder, qsParser: Qs);
    /**
     * Prefix a custom base URL to the final URI
     */
    prefixUrl(url: string): this;
    /**
     * Disable route lookup. Calling this method considers
     * the "identifier" as the route pattern
     */
    disableRouteLookup(): this;
    /**
     * Append query string to the final URI
     */
    qs(queryString?: Record<string, any>): this;
    /**
     * Specify params to apply to the route pattern
     */
    params(params?: any[] | Record<string, any>): this;
    /**
     * Generate URL for the given route identifier. The identifier can be the
     * route name, controller.method name or the route pattern
     * itself.
     */
    make(identifier: string): string;
    /**
     * Generate a signed URL for the given route identifier. The identifier can be the
     * route name, controller.method name or the route pattern
     * itself.
     */
    makeSigned(identifier: string, options?: {
        expiresIn?: string | number;
        purpose?: string;
    }): string;
}

/**
 * Lookup store exposes the API to lookup routes and
 * make URLs for registered routes.
 */
declare class LookupStore {
    #private;
    constructor(encryption: Encryption, qsParser: Qs);
    /**
     * Register route JSON payload
     */
    register(route: RouteJSON): void;
    /**
     * Returns an instance of the URL builder for making
     * route URIs
     */
    builder(): UrlBuilder;
    /**
     * Returns an instance of the URL builder for a specific
     * domain.
     */
    builderForDomain(domain: string): UrlBuilder;
    /**
     * Finds a route by its identifier. The identifier can be the
     * route name, controller.method name or the route pattern
     * itself.
     */
    find(routeIdentifier: string, domain?: string): RouteJSON | null;
    /**
     * Finds a route by its identifier. The identifier can be the
     * route name, controller.method name or the route pattern
     * itself.
     *
     * An error is raised when unable to find the route.
     */
    findOrFail(routeIdentifier: string, domain?: string): RouteJSON;
    /**
     * Check if a route exists. The identifier can be the
     * route name, controller.method name or the route pattern
     * itself.
     */
    has(routeIdentifier: string, domain?: string): boolean;
    toJSON(): {
        [domain: string]: RouteJSON[];
    };
}

/**
 * Shortcut methods for commonly used route matchers
 */
declare class RouteMatchers extends Macroable {
    /**
     * Enforce value to be a number and also casts it to number data
     * type
     */
    number(): {
        match: RegExp;
        cast: (value: string) => number;
    };
    /**
     * Enforce value to be formatted as uuid
     */
    uuid(): {
        match: RegExp;
        cast: (value: string) => string;
    };
    /**
     * Enforce value to be formatted as slug
     */
    slug(): {
        match: RegExp;
    };
}

/**
 * Router class exposes a unified API to register new routes, group them or
 * create route resources.
 *
 * ```ts
 * const router = new Router()
 *
 * router.get('/', async function () {
 *   // handle request
 * })
 * ```
 */
declare class Router extends LookupStore {
    #private;
    /**
     * Collection of routes, including route resource and route
     * group. To get a flat list of routes, call `router.toJSON()`
     */
    routes: (Route | RouteResource | RouteGroup | BriskRoute)[];
    /**
     * A flag to know if routes for explicit domains have been registered.
     * The boolean is computed after calling the "commit" method.
     */
    usingDomains: boolean;
    /**
     * Shortcut methods for commonly used route matchers
     */
    matchers: RouteMatchers;
    constructor(app: Application<any>, encryption: Encryption, qsParser: Qs);
    /**
     * Parses the route pattern
     */
    parsePattern(pattern: string, matchers?: RouteMatchers$1): MatchItRouteToken[];
    /**
     * Define an array of middleware to use on all the routes.
     * Calling this method multiple times pushes to the
     * existing list of middleware
     */
    use(middleware: LazyImport<MiddlewareAsClass>[]): this;
    /**
     * Define a collection of named middleware. The defined collection is
     * not registered anywhere, but instead converted in a new collection
     * of functions you can apply on the routes, or router groups.
     */
    named<NamedMiddleware extends Record<string, LazyImport<MiddlewareAsClass>>>(collection: NamedMiddleware): { [K in keyof NamedMiddleware]: <Args extends GetMiddlewareArgs<UnWrapLazyImport<NamedMiddleware[K]>>>(...args: Args) => {
        name: K;
        args: Args[0];
    } & ParsedGlobalMiddleware; };
    /**
     * Add route for a given pattern and methods
     */
    route<T extends Constructor<any>>(pattern: string, methods: string[], handler: string | RouteFn | [LazyImport<T> | T, GetControllerHandlers<T>?]): Route<T>;
    /**
     * Define a route that handles all common HTTP methods
     */
    any<T extends Constructor<any>>(pattern: string, handler: string | RouteFn | [LazyImport<T> | T, GetControllerHandlers<T>?]): Route<T>;
    /**
     * Define `GET` route
     */
    get<T extends Constructor<any>>(pattern: string, handler: string | RouteFn | [LazyImport<T> | T, GetControllerHandlers<T>?]): Route<T>;
    /**
     * Define `POST` route
     */
    post<T extends Constructor<any>>(pattern: string, handler: string | RouteFn | [LazyImport<T> | T, GetControllerHandlers<T>?]): Route<T>;
    /**
     * Define `PUT` route
     */
    put<T extends Constructor<any>>(pattern: string, handler: string | RouteFn | [LazyImport<T> | T, GetControllerHandlers<T>?]): Route<T>;
    /**
     * Define `PATCH` route
     */
    patch<T extends Constructor<any>>(pattern: string, handler: string | RouteFn | [LazyImport<T> | T, GetControllerHandlers<T>?]): Route<T>;
    /**
     * Define `DELETE` route
     */
    delete<T extends Constructor<any>>(pattern: string, handler: string | RouteFn | [LazyImport<T> | T, GetControllerHandlers<T>?]): Route<T>;
    /**
     * Creates a group of routes. A route group can apply transforms
     * to routes in bulk
     */
    group(callback: () => void): RouteGroup;
    /**
     * Registers a route resource with conventional set of routes
     */
    resource(resource: string, controller: string | LazyImport<Constructor<any>> | Constructor<any>): RouteResource;
    /**
     * Register a route resource with shallow nested routes.
     */
    shallowResource(resource: string, controller: string | LazyImport<Constructor<any>> | Constructor<any>): RouteResource;
    /**
     * Returns a brisk route instance for a given URL pattern
     */
    on(pattern: string): BriskRoute;
    /**
     * Define matcher for a given param. The global params are applied
     * on all the routes (unless overridden at the route level).
     */
    where(param: string, matcher: RouteMatcher | string | RegExp): this;
    /**
     * Commit routes to the store. The router is freezed after the
     * commit method is called.
     */
    commit(): void;
    /**
     * Find route for a given URL, method and optionally domain
     */
    match(url: string, method: string, hostname?: string | null): null | MatchedRoute;
    /**
     * Make URL to a pre-registered route
     */
    makeUrl(routeIdentifier: string, params?: any[] | Record<string, any>, options?: MakeUrlOptions): string;
    /**
     * Makes a signed URL to a pre-registered route.
     */
    makeSignedUrl(routeIdentifier: string, params?: any[] | Record<string, any>, options?: MakeSignedUrlOptions): string;
}

/**
 * Exposes the API to construct redirect routes
 */
declare class Redirect {
    #private;
    constructor(request: IncomingMessage, response: Response, router: Router, qs: Qs);
    /**
     * Set a custom status code.
     */
    status(statusCode: number): this;
    /**
     * Clearing query string values added using the
     * "withQs" method
     */
    clearQs(): this;
    /**
     * Define query string for the redirect. Not passing
     * any value will forward the current request query
     * string.
     */
    withQs(): this;
    withQs(values: Record<string, any>): this;
    withQs(name: string, value: any): this;
    /**
     * Redirect to the previous path.
     */
    back(): void;
    /**
     * Redirect the request using a route identifier.
     */
    toRoute(routeIdentifier: string, params?: any[] | Record<string, any>, options?: MakeUrlOptions): void;
    /**
     * Redirect the request using a path.
     */
    toPath(url: string): void;
}

/**
 * The response is a wrapper over [ServerResponse](https://nodejs.org/api/http.html#http_class_http_serverresponse)
 * streamlining the process of writing response body and automatically setting up appropriate headers.
 */
declare class Response extends Macroable {
    #private;
    request: IncomingMessage;
    response: ServerResponse;
    /**
     * Does response has body set that will written to the
     * response socket at the end of the request
     */
    get hasLazyBody(): boolean;
    /**
     * Find if the response has non-stream content
     */
    get hasContent(): boolean;
    /**
     * Returns true when response body is set using "response.stream"
     * method
     */
    get hasStream(): boolean;
    /**
     * Returns true when response body is set using "response.download"
     * or "response.attachment" methods
     */
    get hasFileToStream(): boolean;
    /**
     * Returns the response content. Check if the response
     * has content using the "hasContent" method
     */
    get content(): [any, boolean, (string | undefined)?] | undefined;
    /**
     * Returns reference to the stream set using "response.stream"
     * method
     */
    get outgoingStream(): stream.Readable | undefined;
    /**
     * Returns reference to the file path set using "response.stream"
     * method.
     */
    get fileToStream(): {
        path: string;
        generateEtag: boolean;
    } | undefined;
    /**
     * Lazy body is used to set the response body. However, do not
     * write it on the socket immediately unless `response.finish`
     * is called.
     */
    lazyBody: Partial<{
        content: [any, boolean, string?];
        stream: [ResponseStream, ((error: NodeJS.ErrnoException) => [string, number?])?];
        fileToStream: [string, boolean, ((error: NodeJS.ErrnoException) => [string, number?])?];
    }>;
    /**
     * The ctx will be set by the context itself. It creates a circular
     * reference
     */
    ctx?: HttpContext;
    constructor(request: IncomingMessage, response: ServerResponse, encryption: Encryption, config: ResponseConfig, router: Router, qs: Qs);
    /**
     * Returns a boolean telling if response is finished or not.
     * Any more attempts to update headers or body will result
     * in raised exceptions.
     */
    get finished(): boolean;
    /**
     * Returns a boolean telling if response headers has been sent or not.
     * Any more attempts to update headers will result in raised
     * exceptions.
     */
    get headersSent(): boolean;
    /**
     * Returns a boolean telling if response headers and body is written
     * or not. When value is `true`, you can feel free to write headers
     * and body.
     */
    get isPending(): boolean;
    /**
     * Writes the body with appropriate response headers. Etag header is set
     * when `generateEtag` is set to `true`.
     *
     * Empty body results in `204`.
     */
    protected writeBody(content: any, generateEtag: boolean, jsonpCallbackName?: string): void;
    /**
     * Stream the body to the response and handles cleaning up the stream
     */
    protected streamBody(body: ResponseStream, errorCallback?: (error: NodeJS.ErrnoException) => [string, number?]): Promise<void>;
    /**
     * Downloads a file by streaming it to the response
     */
    protected streamFileForDownload(filePath: string, generateEtag: boolean, errorCallback?: (error: NodeJS.ErrnoException) => [string, number?]): Promise<void>;
    /**
     * Writes headers with the Node.js res object using the
     * response.setHeader method
     */
    relayHeaders(): void;
    /**
     * Calls res.writeHead on the Node.js res object.
     */
    writeHead(statusCode?: number): this;
    /**
     * Returns the existing value for a given HTTP response
     * header.
     */
    getHeader(key: string): http.OutgoingHttpHeader | undefined;
    /**
     * Get response headers
     */
    getHeaders(): {
        [x: string]: http.OutgoingHttpHeader | undefined;
        accept?: string | string[] | undefined;
        "accept-charset"?: string | string[] | undefined;
        "accept-encoding"?: string | string[] | undefined;
        "accept-language"?: string | string[] | undefined;
        "accept-ranges"?: string | undefined;
        "access-control-allow-credentials"?: string | undefined;
        "access-control-allow-headers"?: string | undefined;
        "access-control-allow-methods"?: string | undefined;
        "access-control-allow-origin"?: string | undefined;
        "access-control-expose-headers"?: string | undefined;
        "access-control-max-age"?: string | undefined;
        "access-control-request-headers"?: string | undefined;
        "access-control-request-method"?: string | undefined;
        age?: string | undefined;
        allow?: string | undefined;
        authorization?: string | undefined;
        "cache-control"?: string | undefined;
        "cdn-cache-control"?: string | undefined;
        connection?: string | string[] | undefined;
        "content-disposition"?: string | undefined;
        "content-encoding"?: string | undefined;
        "content-language"?: string | undefined;
        "content-length"?: string | number | undefined;
        "content-location"?: string | undefined;
        "content-range"?: string | undefined;
        "content-security-policy"?: string | undefined;
        "content-security-policy-report-only"?: string | undefined;
        cookie?: string | string[] | undefined;
        dav?: string | string[] | undefined;
        dnt?: string | undefined;
        date?: string | undefined;
        etag?: string | undefined;
        expect?: string | undefined;
        expires?: string | undefined;
        forwarded?: string | undefined;
        from?: string | undefined;
        host?: string | undefined;
        "if-match"?: string | undefined;
        "if-modified-since"?: string | undefined;
        "if-none-match"?: string | undefined;
        "if-range"?: string | undefined;
        "if-unmodified-since"?: string | undefined;
        "last-modified"?: string | undefined;
        link?: string | string[] | undefined;
        location?: string | undefined;
        "max-forwards"?: string | undefined;
        origin?: string | undefined;
        prgama?: string | string[] | undefined;
        "proxy-authenticate"?: string | string[] | undefined;
        "proxy-authorization"?: string | undefined;
        "public-key-pins"?: string | undefined;
        "public-key-pins-report-only"?: string | undefined;
        range?: string | undefined;
        referer?: string | undefined;
        "referrer-policy"?: string | undefined;
        refresh?: string | undefined;
        "retry-after"?: string | undefined;
        "sec-websocket-accept"?: string | undefined;
        "sec-websocket-extensions"?: string | string[] | undefined;
        "sec-websocket-key"?: string | undefined;
        "sec-websocket-protocol"?: string | string[] | undefined;
        "sec-websocket-version"?: string | undefined;
        server?: string | undefined;
        "set-cookie"?: string | string[] | undefined;
        "strict-transport-security"?: string | undefined;
        te?: string | undefined;
        trailer?: string | undefined;
        "transfer-encoding"?: string | undefined;
        "user-agent"?: string | undefined;
        upgrade?: string | undefined;
        "upgrade-insecure-requests"?: string | undefined;
        vary?: string | undefined;
        via?: string | string[] | undefined;
        warning?: string | undefined;
        "www-authenticate"?: string | string[] | undefined;
        "x-content-type-options"?: string | undefined;
        "x-dns-prefetch-control"?: string | undefined;
        "x-frame-options"?: string | undefined;
        "x-xss-protection"?: string | undefined;
    };
    /**
     * Set header on the response. To `append` values to the existing header, we suggest
     * using [[append]] method.
     *
     * If `value` is non existy, then header won't be set.
     *
     * @example
     * ```js
     * response.header('content-type', 'application/json')
     * ```
     */
    header(key: string, value: CastableHeader): this;
    /**
     * Append value to an existing header. To replace the value, we suggest using
     * [[header]] method.
     *
     * If `value` is not existy, then header won't be set.
     *
     * @example
     * ```js
     * response.append('set-cookie', 'username=virk')
     * ```
     */
    append(key: string, value: CastableHeader): this;
    /**
     * Adds HTTP response header, when it doesn't exists already.
     */
    safeHeader(key: string, value: CastableHeader): this;
    /**
     * Removes the existing response header from being sent.
     */
    removeHeader(key: string): this;
    /**
     * Returns the status code for the response
     */
    getStatus(): number;
    /**
     * Set HTTP status code
     */
    status(code: number): this;
    /**
     * Set's status code only when it's not explictly
     * set
     */
    safeStatus(code: number): this;
    /**
     * Set response type by looking up for the mime-type using
     * partial types like file extensions.
     *
     * Make sure to read [mime-types](https://www.npmjs.com/package/mime-types) docs
     * too.
     *
     * @example
     * ```js
     * response.type('.json') // Content-type: application/json
     * ```
     */
    type(type: string, charset?: string): this;
    /**
     * Set the Vary HTTP header
     */
    vary(field: string | string[]): this;
    /**
     * Set etag by computing hash from the body. This class will set the etag automatically
     * when `etag = true` in the defined config object.
     *
     * Use this function, when you want to compute etag manually for some other resons.
     */
    setEtag(body: any, weak?: boolean): this;
    /**
     * Returns a boolean telling if the new response etag evaluates same
     * as the request header `if-none-match`. In case of `true`, the
     * server must return `304` response, telling the browser to
     * use the client cache.
     *
     * You won't have to deal with this method directly, since AdonisJs will
     * handle this for you when `http.etag = true` inside `config/app.js` file.
     *
     * However, this is how you can use it manually.
     *
     * @example
     * ```js
     * const responseBody = view.render('some-view')
     *
     * // sets the HTTP etag header for response
     * response.setEtag(responseBody)
     *
     * if (response.fresh()) {
     *   response.sendStatus(304)
     * } else {
     *   response.send(responseBody)
     * }
     * ```
     */
    fresh(): boolean;
    /**
     * Returns the response body. Returns null when response
     * body is a stream
     */
    getBody(): any;
    /**
     * Send the body as response and optionally generate etag. The default value
     * is read from `config/app.js` file, using `http.etag` property.
     *
     * This method buffers the body if `explicitEnd = true`, which is the default
     * behavior and do not change, unless you know what you are doing.
     */
    send(body: any, generateEtag?: boolean): void;
    /**
     * Alias of [[send]]
     */
    json(body: any, generateEtag?: boolean): void;
    /**
     * Writes response as JSONP. The callback name is resolved as follows, with priority
     * from top to bottom.
     *
     * 1. Explicitly defined as 2nd Param.
     * 2. Fetch from request query string.
     * 3. Use the config value `http.jsonpCallbackName` from `config/app.js`.
     * 4. Fallback to `callback`.
     *
     * This method buffers the body if `explicitEnd = true`, which is the default
     * behavior and do not change, unless you know what you are doing.
     */
    jsonp(body: any, callbackName?: string, generateEtag?: boolean): void;
    /**
     * Pipe stream to the response. This method will gracefully destroy
     * the stream, avoiding memory leaks.
     *
     * If `raiseErrors=false`, then this method will self handle all the exceptions by
     * writing a generic HTTP response. To have more control over the error, it is
     * recommended to set `raiseErrors=true` and wrap this function inside a
     * `try/catch` statement.
     *
     * Streaming a file from the disk and showing 404 when file is missing.
     *
     * @example
     * ```js
     * // Errors handled automatically with generic HTTP response
     * response.stream(fs.createReadStream('file.txt'))
     *
     * // Manually handle (note the await call)
     * try {
     *   await response.stream(fs.createReadStream('file.txt'))
     * } catch () {
     *   response.status(404).send('File not found')
     * }
     * ```
     */
    stream(body: ResponseStream, errorCallback?: (error: NodeJS.ErrnoException) => [string, number?]): void;
    /**
     * Download file by streaming it from the file path. This method will setup
     * appropriate `Content-type`, `Content-type` and `Last-modified` headers.
     *
     * Unexpected stream errors are handled gracefully to avoid memory leaks.
     *
     * If `raiseErrors=false`, then this method will self handle all the exceptions by
     * writing a generic HTTP response. To have more control over the error, it is
     * recommended to set `raiseErrors=true` and wrap this function inside a
     * `try/catch` statement.
     *
     * @example
     * ```js
     * // Errors handled automatically with generic HTTP response
     * response.download('somefile.jpg')
     *
     * // Manually handle (note the await call)
     * try {
     *   await response.download('somefile.jpg')
     * } catch (error) {
     *   response.status(error.code === 'ENOENT' ? 404 : 500)
     *   response.send('Cannot process file')
     * }
     * ```
     */
    download(filePath: string, generateEtag?: boolean, errorCallback?: (error: NodeJS.ErrnoException) => [string, number?]): void;
    /**
     * Download the file by forcing the user to save the file vs displaying it
     * within the browser.
     *
     * Internally calls [[download]]
     */
    attachment(filePath: string, name?: string, disposition?: string, generateEtag?: boolean, errorCallback?: (error: NodeJS.ErrnoException) => [string, number?]): void;
    /**
     * Set the location header.
     *
     * @example
     * ```js
     * response.location('/login')
     * ```
     */
    location(url: string): this;
    /**
     * Redirect the request.
     *
     * @example
     * ```js
     * response.redirect('/foo')
     * response.redirect().toRoute('foo.bar')
     * response.redirect().back()
     * ```
     */
    redirect(): Redirect;
    redirect(path: string, forwardQueryString?: boolean, statusCode?: number): void;
    /**
     * Abort the request with custom body and a status code. 400 is
     * used when status is not defined
     */
    abort(body: any, status?: number): never;
    /**
     * Abort the request with custom body and a status code when
     * passed condition returns `true`
     */
    abortIf(condition: unknown, body: any, status?: number): asserts condition is undefined | null | false;
    /**
     * Abort the request with custom body and a status code when
     * passed condition returns `false`
     */
    abortUnless<T>(condition: T, body: any, status?: number): asserts condition is Exclude<T, undefined | null | false>;
    /**
     * Set signed cookie as the response header. The inline options overrides
     * all options from the config.
     */
    cookie(key: string, value: any, options?: Partial<CookieOptions>): this;
    /**
     * Set encrypted cookie as the response header. The inline options overrides
     * all options from the config.
     */
    encryptedCookie(key: string, value: any, options?: Partial<CookieOptions>): this;
    /**
     * Set unsigned cookie as the response header. The inline options overrides
     * all options from the config.
     */
    plainCookie(key: string, value: any, options?: Partial<CookieOptions & {
        encode: boolean;
    }>): this;
    /**
     * Clear existing cookie.
     */
    clearCookie(key: string, options?: Partial<CookieOptions>): this;
    /**
     * Finishes the response by writing the lazy body, when `explicitEnd = true`
     * and response is already pending.
     *
     * Calling this method twice or when `explicitEnd = false` is noop.
     */
    finish(): void;
    /**
     * Shorthand method to finish request with "100" status code
     */
    continue(): void;
    /**
     * Shorthand method to finish request with "101" status code
     */
    switchingProtocols(): void;
    /**
     * Shorthand method to finish request with "200" status code
     */
    ok(body: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "201" status code
     */
    created(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "202" status code
     */
    accepted(body: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "203" status code
     */
    nonAuthoritativeInformation(body: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "204" status code
     */
    noContent(): void;
    /**
     * Shorthand method to finish request with "205" status code
     */
    resetContent(): void;
    /**
     * Shorthand method to finish request with "206" status code
     */
    partialContent(body: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "300" status code
     */
    multipleChoices(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "301" status code
     */
    movedPermanently(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "302" status code
     */
    movedTemporarily(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "303" status code
     */
    seeOther(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "304" status code
     */
    notModified(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "305" status code
     */
    useProxy(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "307" status code
     */
    temporaryRedirect(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "400" status code
     */
    badRequest(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "401" status code
     */
    unauthorized(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "402" status code
     */
    paymentRequired(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "403" status code
     */
    forbidden(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "404" status code
     */
    notFound(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "405" status code
     */
    methodNotAllowed(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "406" status code
     */
    notAcceptable(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "407" status code
     */
    proxyAuthenticationRequired(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "408" status code
     */
    requestTimeout(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "409" status code
     */
    conflict(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "401" status code
     */
    gone(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "411" status code
     */
    lengthRequired(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "412" status code
     */
    preconditionFailed(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "413" status code
     */
    requestEntityTooLarge(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "414" status code
     */
    requestUriTooLong(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "415" status code
     */
    unsupportedMediaType(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "416" status code
     */
    requestedRangeNotSatisfiable(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "417" status code
     */
    expectationFailed(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "422" status code
     */
    unprocessableEntity(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "429" status code
     */
    tooManyRequests(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "500" status code
     */
    internalServerError(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "501" status code
     */
    notImplemented(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "502" status code
     */
    badGateway(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "503" status code
     */
    serviceUnavailable(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "504" status code
     */
    gatewayTimeout(body?: any, generateEtag?: boolean): void;
    /**
     * Shorthand method to finish request with "505" status code
     */
    httpVersionNotSupported(body?: any, generateEtag?: boolean): void;
}

/**
 * Http context encapsulates properties for a given HTTP request. The
 * context class can be extended using macros and getters.
 */
declare class HttpContext extends Macroable {
    request: Request;
    response: Response;
    logger: Logger;
    containerResolver: ContainerResolver<any>;
    /**
     * Find if async localstorage is enabled for HTTP requests
     * or not
     */
    static get usingAsyncLocalStorage(): boolean;
    /**
     * Get access to the HTTP context. Available only when
     * "usingAsyncLocalStorage" is true
     */
    static get(): HttpContext | null;
    /**
     * Get the HttpContext instance or raise an exception if not
     * available
     */
    static getOrFail(): HttpContext;
    /**
     * Run a method that doesn't have access to HTTP context from
     * the async local storage.
     */
    static runOutsideContext<T>(callback: (...args: any[]) => T, ...args: any[]): T;
    /**
     * Reference to the current route. Not available inside
     * server middleware
     */
    route?: StoreRouteNode;
    /**
     * A unique key for the current route
     */
    routeKey?: string;
    /**
     * Route params
     */
    params: Record<string, any>;
    /**
     * Route subdomains
     */
    subdomains: Record<string, any>;
    constructor(request: Request, response: Response, logger: Logger, containerResolver: ContainerResolver<any>);
    /**
     * A helper to see top level properties on the context object
     */
    inspect(): string;
}

/**
 * HTTP Request class exposes the interface to consistently read values
 * related to a given HTTP request. The class is wrapper over
 * [IncomingMessage](https://nodejs.org/api/http.html#http_class_http_incomingmessage)
 * and has extended API.
 *
 * You can access the original [IncomingMessage](https://nodejs.org/api/http.html#http_class_http_incomingmessage)
 * using `request.request` property.
 */
declare class Request extends Macroable {
    #private;
    request: IncomingMessage;
    response: ServerResponse;
    /**
     * Parses copy of the URL with query string as a string and not
     * object. This is done to build URL's with query string without
     * stringifying the object
     */
    parsedUrl: UrlWithStringQuery;
    /**
     * The ctx will be set by the context itself. It creates a circular
     * reference
     */
    ctx?: HttpContext;
    constructor(request: IncomingMessage, response: ServerResponse, encryption: Encryption, config: RequestConfig, qsParser: Qs);
    /**
     * Returns the request id from the `x-request-id` header. The
     * header is untouched, if it already exists.
     */
    id(): string | undefined;
    /**
     * Set initial request body. A copy of the input will be maintained as the original
     * request body. Since the request body and query string is subject to mutations, we
     * keep one original reference to flash old data (whenever required).
     *
     * This method is supposed to be invoked by the body parser and must be called only
     * once. For further mutations make use of `updateBody` method.
     */
    setInitialBody(body: Record<string, any>): void;
    /**
     * Update the request body with new data object. The `all` property
     * will be re-computed by merging the query string and request
     * body.
     */
    updateBody(body: Record<string, any>): void;
    /**
     * Update the request raw body. Bodyparser sets this when unable to parse
     * the request body or when request is multipart/form-data.
     */
    updateRawBody(rawBody: string): void;
    /**
     * Update the query string with the new data object. The `all` property
     * will be re-computed by merging the query and the request body.
     */
    updateQs(data: Record<string, any>): void;
    /**
     * Returns route params
     */
    params(): Record<string, any>;
    /**
     * Returns the query string object by reference
     */
    qs(): Record<string, any>;
    /**
     * Returns reference to the request body
     */
    body(): Record<string, any>;
    /**
     * Returns reference to the merged copy of request body
     * and query string
     */
    all(): Record<string, any>;
    /**
     * Returns reference to the merged copy of original request
     * query string and body
     */
    original(): Record<string, any>;
    /**
     * Returns the request raw body (if exists), or returns `null`.
     *
     * Ideally you must be dealing with the parsed body accessed using [[input]], [[all]] or
     * [[post]] methods. The `raw` body is always a string.
     */
    raw(): string | null;
    /**
     * Returns value for a given key from the request body or query string.
     * The `defaultValue` is used when original value is `undefined`.
     *
     * @example
     * ```js
     * request.input('username')
     *
     * // with default value
     * request.input('username', 'virk')
     * ```
     */
    input(key: string, defaultValue?: any): any;
    /**
     * Returns value for a given key from route params
     *
     * @example
     * ```js
     * request.param('id')
     *
     * // with default value
     * request.param('id', 1)
     * ```
     */
    param(key: string, defaultValue?: any): any;
    /**
     * Get everything from the request body except the given keys.
     *
     * @example
     * ```js
     * request.except(['_csrf'])
     * ```
     */
    except(keys: string[]): Record<string, any>;
    /**
     * Get value for specified keys.
     *
     * @example
     * ```js
     * request.only(['username', 'age'])
     * ```
     */
    only<T extends string>(keys: T[]): {
        [K in T]: any;
    };
    /**
     * Returns the HTTP request method. This is the original
     * request method. For spoofed request method, make
     * use of [[method]].
     *
     * @example
     * ```js
     * request.intended()
     * ```
     */
    intended(): string;
    /**
     * Returns the request HTTP method by taking method spoofing into account.
     *
     * Method spoofing works when all of the following are true.
     *
     * 1. `app.http.allowMethodSpoofing` config value is true.
     * 2. request query string has `_method`.
     * 3. The [[intended]] request method is `POST`.
     *
     * @example
     * ```js
     * request.method()
     * ```
     */
    method(): string;
    /**
     * Returns a copy of headers as an object
     */
    headers(): IncomingHttpHeaders;
    /**
     * Returns value for a given header key. The default value is
     * used when original value is `undefined`.
     */
    header(key: string, defaultValue?: any): string | undefined;
    /**
     * Returns the ip address of the user. This method is optimize to fetch
     * ip address even when running your AdonisJs app behind a proxy.
     *
     * You can also define your own custom function to compute the ip address by
     * defining `app.http.getIp` as a function inside the config file.
     *
     * ```js
     * {
     *   http: {
     *     getIp (request) {
     *       // I am using nginx as a proxy server and want to trust 'x-real-ip'
     *       return request.header('x-real-ip')
     *     }
     *   }
     * }
     * ```
     *
     * You can control the behavior of trusting the proxy values by defining it
     * inside the `config/app.js` file.
     *
     * ```js
     * {
     *   http: {
     *    trustProxy: '127.0.0.1'
     *   }
     * }
     * ```
     *
     * The value of trustProxy is passed directly to [proxy-addr](https://www.npmjs.com/package/proxy-addr)
     */
    ip(): string;
    /**
     * Returns an array of ip addresses from most to least trusted one.
     * This method is optimize to fetch ip address even when running
     * your AdonisJs app behind a proxy.
     *
     * You can control the behavior of trusting the proxy values by defining it
     * inside the `config/app.js` file.
     *
     * ```js
     * {
     *   http: {
     *    trustProxy: '127.0.0.1'
     *   }
     * }
     * ```
     *
     * The value of trustProxy is passed directly to [proxy-addr](https://www.npmjs.com/package/proxy-addr)
     */
    ips(): string[];
    /**
     * Returns the request protocol by checking for the URL protocol or
     * `X-Forwarded-Proto` header.
     *
     * If the `trust` is evaluated to `false`, then URL protocol is returned,
     * otherwise `X-Forwarded-Proto` header is used (if exists).
     *
     * You can control the behavior of trusting the proxy values by defining it
     * inside the `config/app.js` file.
     *
     * ```js
     * {
     *   http: {
     *    trustProxy: '127.0.0.1'
     *   }
     * }
     * ```
     *
     * The value of trustProxy is passed directly to [proxy-addr](https://www.npmjs.com/package/proxy-addr)
     */
    protocol(): string;
    /**
     * Returns a boolean telling if request is served over `https`
     * or not. Check [[protocol]] method to know how protocol is
     * fetched.
     */
    secure(): boolean;
    /**
     * Returns the request host. If proxy headers are trusted, then
     * `X-Forwarded-Host` is given priority over the `Host` header.
     *
     * You can control the behavior of trusting the proxy values by defining it
     * inside the `config/app.js` file.
     *
     * ```js
     * {
     *   http: {
     *    trustProxy: '127.0.0.1'
     *   }
     * }
     * ```
     *
     * The value of trustProxy is passed directly to [proxy-addr](https://www.npmjs.com/package/proxy-addr)
     */
    host(): string | null;
    /**
     * Returns the request hostname. If proxy headers are trusted, then
     * `X-Forwarded-Host` is given priority over the `Host` header.
     *
     * You can control the behavior of trusting the proxy values by defining it
     * inside the `config/app.js` file.
     *
     * ```js
     * {
     *   http: {
     *    trustProxy: '127.0.0.1'
     *   }
     * }
     * ```
     *
     * The value of trustProxy is passed directly to [proxy-addr](https://www.npmjs.com/package/proxy-addr)
     */
    hostname(): string | null;
    /**
     * Returns an array of subdomains for the given host. An empty array is
     * returned if [[hostname]] is `null` or is an IP address.
     *
     * Also `www` is not considered as a subdomain
     */
    subdomains(): string[];
    /**
     * Returns a boolean telling, if request `X-Requested-With === 'xmlhttprequest'`
     * or not.
     */
    ajax(): boolean;
    /**
     * Returns a boolean telling, if request has `X-Pjax` header
     * set or not
     */
    pjax(): boolean;
    /**
     * Returns the request relative URL.
     *
     * @example
     * ```js
     * request.url()
     *
     * // include query string
     * request.url(true)
     * ```
     */
    url(includeQueryString?: boolean): string;
    /**
     * Returns the complete HTTP url by combining
     * [[protocol]]://[[hostname]]/[[url]]
     *
     * @example
     * ```js
     * request.completeUrl()
     *
     * // include query string
     * request.completeUrl(true)
     * ```
     */
    completeUrl(includeQueryString?: boolean): string;
    /**
     * Find if the current HTTP request is for the given route or the routes
     */
    matchesRoute(routeIdentifier: string | string[]): boolean;
    /**
     * Returns the best matching content type of the request by
     * matching against the given types.
     *
     * The content type is picked from the `content-type` header and request
     * must have body.
     *
     * The method response highly depends upon the types array values. Described below:
     *
     * | Type(s) | Return value |
     * |----------|---------------|
     * | ['json'] | json |
     * | ['application/*'] | application/json |
     * | ['vnd+json'] | application/json |
     *
     * @example
     * ```js
     * const bodyType = request.is(['json', 'xml'])
     *
     * if (bodyType === 'json') {
     *  // process JSON
     * }
     *
     * if (bodyType === 'xml') {
     *  // process XML
     * }
     * ```
     */
    is(types: string[]): string | null;
    /**
     * Returns the best type using `Accept` header and
     * by matching it against the given types.
     *
     * If nothing is matched, then `null` will be returned
     *
     * Make sure to check [accepts](https://www.npmjs.com/package/accepts) package
     * docs too.
     *
     * @example
     * ```js
     * switch (request.accepts(['json', 'html'])) {
     *   case 'json':
     *     return response.json(user)
     *   case 'html':
     *     return view.render('user', { user })
     *   default:
     *     // decide yourself
     * }
     * ```
     */
    accepts<T extends string>(types: T[]): T | null;
    /**
     * Return the types that the request accepts, in the order of the
     * client's preference (most preferred first).
     *
     * Make sure to check [accepts](https://www.npmjs.com/package/accepts) package
     * docs too.
     */
    types(): string[];
    /**
     * Returns the best language using `Accept-language` header
     * and by matching it against the given languages.
     *
     * If nothing is matched, then `null` will be returned
     *
     * Make sure to check [accepts](https://www.npmjs.com/package/accepts) package
     * docs too.
     *
     * @example
     * ```js
     * switch (request.language(['fr', 'de'])) {
     *   case 'fr':
     *     return view.render('about', { lang: 'fr' })
     *   case 'de':
     *     return view.render('about', { lang: 'de' })
     *   default:
     *     return view.render('about', { lang: 'en' })
     * }
     * ```
     */
    language<T extends string>(languages: T[]): T | null;
    /**
     * Return the languages that the request accepts, in the order of the
     * client's preference (most preferred first).
     *
     * Make sure to check [accepts](https://www.npmjs.com/package/accepts) package
     * docs too.
     */
    languages(): string[];
    /**
     * Returns the best charset using `Accept-charset` header
     * and by matching it against the given charsets.
     *
     * If nothing is matched, then `null` will be returned
     *
     * Make sure to check [accepts](https://www.npmjs.com/package/accepts) package
     * docs too.
     *
     * @example
     * ```js
     * switch (request.charset(['utf-8', 'ISO-8859-1'])) {
     *   case 'utf-8':
     *     // make utf-8 friendly response
     *   case 'ISO-8859-1':
     *     // make ISO-8859-1 friendly response
     * }
     * ```
     */
    charset<T extends string>(charsets: T[]): T | null;
    /**
     * Return the charsets that the request accepts, in the order of the
     * client's preference (most preferred first).
     *
     * Make sure to check [accepts](https://www.npmjs.com/package/accepts) package
     * docs too.
     */
    charsets(): string[];
    /**
     * Returns the best encoding using `Accept-encoding` header
     * and by matching it against the given encodings.
     *
     * If nothing is matched, then `null` will be returned
     *
     * Make sure to check [accepts](https://www.npmjs.com/package/accepts) package
     * docs too.
     */
    encoding<T extends string>(encodings: T[]): T | null;
    /**
     * Return the charsets that the request accepts, in the order of the
     * client's preference (most preferred first).
     *
     * Make sure to check [accepts](https://www.npmjs.com/package/accepts) package
     * docs too.
     */
    encodings(): string[];
    /**
     * Returns a boolean telling if request has body
     */
    hasBody(): boolean;
    /**
     * Returns a boolean telling if the new response etag evaluates same
     * as the request header `if-none-match`. In case of `true`, the
     * server must return `304` response, telling the browser to
     * use the client cache.
     *
     * You won't have to deal with this method directly, since AdonisJs will
     * handle this for you when `http.etag = true` inside `config/app.js` file.
     *
     * However, this is how you can use it manually.
     *
     * ```js
     * const responseBody = view.render('some-view')
     *
     * // sets the HTTP etag header for response
     * response.setEtag(responseBody)
     *
     * if (request.fresh()) {
     *   response.sendStatus(304)
     * } else {
     *   response.send(responseBody)
     * }
     * ```
     */
    fresh(): boolean;
    /**
     * Opposite of [[fresh]]
     */
    stale(): boolean;
    /**
     * Returns all parsed and signed cookies. Signed cookies ensures
     * that their value isn't tampered.
     */
    cookiesList(): Record<string, any>;
    /**
     * Returns value for a given key from signed cookies. Optional
     * defaultValue is returned when actual value is undefined.
     */
    cookie(key: string, defaultValue?: string): any;
    /**
     * Returns value for a given key from signed cookies. Optional
     * defaultValue is returned when actual value is undefined.
     */
    encryptedCookie(key: string, defaultValue?: string): any;
    /**
     * Returns value for a given key from unsigned cookies. Optional
     * defaultValue is returned when actual value is undefined.
     */
    plainCookie(key: string, options?: {
        defaultValue?: string;
        encoded?: boolean;
    }): any;
    plainCookie(key: string, defaultValue?: string, encoded?: boolean): any;
    /**
     * Returns a boolean telling if a signed url as a valid signature
     * or not.
     */
    hasValidSignature(purpose?: string): boolean;
    /**
     * Serializes request to JSON format
     */
    serialize(): {
        id: string | undefined;
        url: string;
        query: string | null;
        body: Record<string, any>;
        params: Record<string, any>;
        headers: IncomingHttpHeaders;
        method: string;
        protocol: string;
        cookies: Record<string, any>;
        hostname: string | null;
        ip: string;
        subdomains: Record<string, any>;
    };
    /**
     * toJSON copy of the request
     */
    toJSON(): {
        id: string | undefined;
        url: string;
        query: string | null;
        body: Record<string, any>;
        params: Record<string, any>;
        headers: IncomingHttpHeaders;
        method: string;
        protocol: string;
        cookies: Record<string, any>;
        hostname: string | null;
        ip: string;
        subdomains: Record<string, any>;
    };
}

export { MatchedRoute as A, BriskRoute as B, Constructor as C, RouteMatcher as D, ErrorHandlerAsAClass as E, RouteMatchers$1 as F, GetMiddlewareArgs as G, HttpContext as H, RouteJSON as I, ResourceActionNames as J, MakeUrlOptions as K, LazyImport as L, MiddlewareAsClass as M, MakeSignedUrlOptions as N, OneOrMore as O, ParsedGlobalMiddleware as P, QSParserConfig as Q, Router as R, ServerConfig as S, TestingMiddlewarePipeline as T, UnWrapLazyImport as U, HttpRequestFinishedPayload as V, ServerErrorHandler as W, Request as a, Response as b, StatusPageRange as c, StatusPageRenderer as d, HttpError as e, Redirect as f, Route as g, RouteGroup as h, RouteResource as i, Qs as j, RequestConfig as k, ResponseConfig as l, MiddlewareFn as m, ParsedNamedMiddleware as n, CookieOptions as o, CastableHeader as p, ResponseStream as q, GetControllerHandlers as r, MatchItRouteToken as s, RouteFn as t, StoreRouteHandler as u, StoreRouteMiddleware as v, StoreRouteNode as w, StoreMethodNode as x, StoreDomainNode as y, StoreRoutesTree as z };
